﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Web.UI;

namespace MvcJqTGrid.Extensions {

  /// <summary>
  /// 
  /// </summary>
  public enum SortDirection {
    /// <summary>
    /// 
    /// </summary>
    Asc = 1,
    /// <summary>
    /// 
    /// </summary>
    Desc = 2
  }

  /// <summary>
  /// LieberLieber Grid for ASP.NET MVC is a view component which uses jqGrid as bases
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public class Grid<T> where T : class {

    private readonly HtmlTextWriter _htmlTextWriter;
    private string _gridId;
    private string _caption;
    private string _dataUrl;
    private string _postType;
    private string _pagerId;
    private int _rowsPerPage;
    private string _rowList;
    private string _sortName;
    private SortDirection _sortOrder;
    private bool _viewRecords;
    private string _imagePath;
    private int _columnWidth;
    private string _columnAlign;
    private string _tableCssClass;
    private string _pagerCssClass;
    private string _dataType;
    private bool _autowidth;

    /// <summary>
    /// Gets or sets the name.
    /// </summary>
    /// <value>The name.</value>
    public string GridId {
      get {
        return _gridId;
      }

      set {
        if (string.IsNullOrEmpty(value))
          throw new ArgumentException("GridId can not be null or empty!");

        _gridId = value;
      }
    }



    public Grid(string gridId, TextWriter textWriter){
      //second argument removes the tabstring so it is less html code produced!
      _htmlTextWriter = new HtmlTextWriter(textWriter,""); 
      GridId = gridId;
      _dataUrl =  "/DynamicGridData/";
      _postType = "POST";
      _dataType = "json";
      _pagerId = "pager";
      _rowsPerPage = 10;
      _rowList = "5, 10, 20, 50";
      _sortName = "Id";
      _sortOrder = SortDirection.Asc;
      _viewRecords = true;
      _imagePath = "";
      _caption = "";
      _columnWidth = 0;
      _columnAlign = "left";
      _tableCssClass = "scroll";
      _pagerCssClass = "scroll";
      _autowidth = false;
    }

    public Grid<T> Caption(string caption)
    {
      _caption = caption;
      return this;
    }

    public Grid<T> DataUrl(string dataUrl)
    {
      _dataUrl = dataUrl;
      return this;
    }

    public Grid<T> PostType(string postType)
    {
      _postType = postType;
      return this;
    }

    public Grid<T> PagerId(string pagerId) {
      _pagerId = pagerId;
      return this;
    }

    public Grid<T> RowsPerPage(int rowsPerPage) {
      _rowsPerPage = rowsPerPage;
      return this;
    }

    public Grid<T> RowList(IList<int> rowList) {
      _rowList = string.Join(",", rowList);
      return this;
    }

    public Grid<T> SortName(string sortName) {
      _sortName = sortName;
      return this;
    }

    public Grid<T> SortOrder(SortDirection sortOrder) {
      _sortOrder = sortOrder;
      return this;
    }

    public Grid<T> ViewRecords(bool viewRecords) {
      _viewRecords = viewRecords;
      return this;
    }
    public Grid<T> ImagePath(string imagePath) {
      _imagePath = imagePath;
      return this;
    }

    public Grid<T> ColumnWidth(int columnWidth) {
      _columnWidth = columnWidth;
      return this;
    }

    public Grid<T> ColumnWidth(string columnAlign) {
      _columnAlign = columnAlign;
      return this;
    }

    public Grid<T> TableCssClass(string tableCssClass) {
      _tableCssClass = tableCssClass;
      return this;
    }

    public Grid<T> PagerCssClass(string pagerCssClass) {
      _pagerCssClass = pagerCssClass;
      return this;
    }

    public Grid<T> DataType(string dataType) {
      _dataType = dataType;
      return this;
    }

    public Grid<T> AutoWidth(bool autowidth) {
      _autowidth = autowidth;
      return this;
    }

    public override string ToString() {
      Render();
      return null;
    }

    /// <summary>
    /// Renders the JqGrid java script.
    /// </summary>
    private void RenderJqGridJavaScript(){
      _htmlTextWriter.AddAttribute(HtmlTextWriterAttribute.Type, "text/javascript");
      _htmlTextWriter.AddAttribute("language","javascript");
      _htmlTextWriter.RenderBeginTag(HtmlTextWriterTag.Script);
      _htmlTextWriter.Write(BuildJqGrid());
      _htmlTextWriter.RenderEndTag();
     
     // htmlTextWriter.RenderEndTag();
    }

    /// <summary>
    /// Renders the jqGridTable.
    /// </summary>
    private void RenderJqGridTable(){
      _htmlTextWriter.AddAttribute("id", _gridId);
      _htmlTextWriter.AddAttribute("class", _tableCssClass);
      _htmlTextWriter.AddAttribute("cellpadding", "0");
      _htmlTextWriter.AddAttribute("cellspacing", "0");
      _htmlTextWriter.RenderBeginTag(HtmlTextWriterTag.Table); 
      _htmlTextWriter.RenderEndTag();
    }

    private void RenderJqGridPager(){
      _htmlTextWriter.AddAttribute("id", _pagerId);
      _htmlTextWriter.AddAttribute("class", _pagerCssClass);
      _htmlTextWriter.AddAttribute("style", "text-align:center;");
      _htmlTextWriter.RenderBeginTag(HtmlTextWriterTag.Div);
      _htmlTextWriter.RenderEndTag();

    }


    /// <summary>
    /// Builds the jqgrid.
    /// </summary>
    /// <returns></returns>
    private string BuildJqGrid(){
      StringBuilder sbGrid = new StringBuilder();
      sbGrid.AppendLine("jQuery(document).ready(function() {");
      sbGrid.AppendFormat("jQuery(\"#{0}\")",_gridId).AppendLine();
      sbGrid.AppendFormat(".jqGrid({{url: '{0}',",_dataUrl).AppendLine();
      sbGrid.AppendFormat("mtype: '{0}',",_postType).AppendLine();
      sbGrid.AppendFormat("datatype: '{0}',",_dataType).AppendLine();
      GetColumnNames(sbGrid);
      GetColumnModel(sbGrid);
      sbGrid.AppendFormat("pager: jQuery('#{0}'),", _pagerId).AppendLine();
      sbGrid.AppendFormat("rowNum:{0},", _rowsPerPage).AppendLine();
      sbGrid.AppendFormat("rowList:[{0}],", _rowList).AppendLine();
      sbGrid.AppendFormat("sortname:'{0}',", _sortName).AppendLine();
      sbGrid.AppendFormat("sortorder:'{0}',", _sortOrder).AppendLine();
      sbGrid.AppendFormat("viewrecords:{0},", _viewRecords.ToString().ToLower()).AppendLine();
      sbGrid.AppendFormat("imgpath:'{0}',", _imagePath).AppendLine();
      sbGrid.AppendFormat("caption:'{0}',", _caption).AppendLine();
      sbGrid.AppendFormat("autowidth:{0},", _autowidth.ToString().ToLower()).AppendLine();
      sbGrid.AppendLine("});");
      sbGrid.AppendLine("});");
      return sbGrid.ToString();
    }


    /// <summary>
    /// Gets the column model.
    /// </summary>
    /// <param name="sb">The sb.</param>
    /// <returns></returns>
    private static void GetColumnModel(StringBuilder sb) {
      // colNames: ['Id', 'Votes', 'Title'],");
      Type objType = typeof(T);

      sb.AppendLine("colNames: [");

      foreach (var property in objType.GetProperties()) {
        sb.AppendFormat("'{0}',", property.Name);
      }
      sb.Remove(sb.Length - 1, 1); //remove the last " , "
      sb.Append("],");
    }


    /// <summary>
    /// Gets the column names.
    /// </summary>
    /// <param name="sb">The sb.</param>
    /// <returns></returns>
    private void GetColumnNames(StringBuilder sb) {
      // { name: 'Id', index: 'Id', width: 40, align: 'left' },");
      Type objType = typeof(T);

      sb.AppendLine("colModel: [");

      foreach (var property in objType.GetProperties()) {
        sb.AppendFormat("{{ name: '{0}', index: '{0}', width: {1}, align: '{2}' }},", property.Name,_columnWidth,_columnAlign);
      }
      sb.Remove(sb.Length - 1, 1); //remove the last " , "
      sb.Append("],");
    }

    /// <summary>
    /// Renders the JqGrid
    /// </summary>
    private void Render() {
      _htmlTextWriter.BeginRender();
      RenderJqGridJavaScript();
      RenderJqGridTable();
      RenderJqGridPager();
      _htmlTextWriter.EndRender();
      _htmlTextWriter.Close();
    }
  }
}
