﻿using System;
using System.Linq;
using System.Web.Mvc;

namespace MvcJqTGrid.Extensions {
  public static  class GridExtension  {

    /// <summary>
    /// Creates the json grid result
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="source">The source.</param>
    /// <param name="column">The column.</param>
    /// <param name="sortOrder">The sort order.</param>
    /// <param name="page">The page.</param>
    /// <param name="pageSize">Size of the page.</param>
    /// <returns></returns>
    public static JsonResult AsJqGridResult<T>(this IQueryable<T> source, string column, string sortOrder, int page, int pageSize)
    {
      int pageIndex = Convert.ToInt32(page) - 1;
      int totalRecords = source.Count();
      int totalPages = (int) Math.Ceiling((float) totalRecords/(float) pageSize);

      var list = (sortOrder.ToLower()=="asc") ? 
          
          source.OrderBy(new Func<T, IComparable>(item => (IComparable) GetPropertyValue(item, column))).Skip(pageIndex*pageSize).Take(pageSize) :
          
          source.OrderByDescending(new Func<T, IComparable>(item => (IComparable) GetPropertyValue(item, column))).Skip(pageIndex*pageSize).Take(pageSize);
      

      var jsonData = new
      {
        total = totalPages,
        page,
        records = totalRecords,
        rows = (
                from item in list
                select new
                          {
                            i = Guid.NewGuid(),
                            //question.Id,
                            cell = GetGridCells(item)
                          }).ToArray()
      };

      return new JsonResult {Data = jsonData};
    }

    /// <summary>
    /// Gets the property value.
    /// </summary>
    /// <param name="obj">The obj.</param>
    /// <param name="property">The property.</param>
    /// <returns></returns>
    private static object GetPropertyValue(object obj, string property) {
      System.Reflection.PropertyInfo propertyInfo = obj.GetType().GetProperty(property);
      return propertyInfo.GetValue(obj, null);
    }

    /// <summary>
    /// Gets the grid cells.
    /// </summary>
    /// <param name="dataObject">The data object.</param>
    /// <returns></returns>
    private static string[] GetGridCells(object dataObject)
    {
      Type objType = dataObject.GetType();
      return objType.GetProperties().Select(property => property.GetValue(dataObject, null).ToString()).ToArray();
    }
  }
}
